unit HDUS_BDA;

// ----------------------------------------------
//   HDUS_BDA
//   MobileHackerz : http://mobilehackerz.jp
// ----------------------------------------------
(* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *  This Program is free software; you can redistribute it and/or modify     *
 *  it under the terms of the GNU General Public License as published by     *
 *  the Free Software Foundation; either version 2, or (at your option)      *
 *  any later version.                                                       *
 *                                                                           *
 *  This Program is distributed in the hope that it will be useful,          *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of           *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             *
 *  GNU General Public License for more details.                             *
 *                                                                           *
 *  You should have received a copy of the GNU General Public License        *
 *  along with GNU Make; see the file COPYING.  If not, write to             *
 *  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.    *
 *  http://www.gnu.org/copyleft/gpl.html                                     *
 *                                                                           *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *)


interface

uses
  Windows, Messages, SysUtils, Variants, Classes,
  BDADriver, ActiveX, DCDVB_SDK_TuningPlugins;

type
  TLogMethod   = procedure (AText:String) of Object;
  TTSMethod    = procedure (ABuffer: PByte; ASize: Integer) of Object;

  THDUS_Driver = class(TInterfacedObject,IDCDVBTuningPluginApplicationCallback)
  private
    { Private 錾 }
    FTuner     :TDCDVBTuningPlugin;
    FBDA       :TDCDVBTuningPluginDevice;
    FLogMethod :TLogMethod;
    FTSMethod  :TTSMethod;
    FFrequency :Integer;
    FBandWidth :Integer;
    function GetTunerCount: Integer;
    function GetTunerName(idx: Integer): String;
    function GetTunerID(idx: Integer): String;
  public
    { Public 錾 }
    constructor Create;
    destructor  Destroy; override;
    function Activate(ID:String): Boolean;
    function Tune(Freq:Integer; Band:Integer): Boolean;
    function Deactivate: Boolean;
    function ReceiveTSStream(ABuffer: PByte; ASize: Integer): Integer; stdcall;
    function LogEvent(AMethod: PChar; AText: PChar): Integer; stdcall;
    property LogMethod  :TLogMethod read FLogMethod  write FLogMethod;
    property TSMethod   :TTSMethod  read FTSMethod   write FTSMethod;
    property TunerCount :Integer    read GetTunerCount;
    property TunerName[idx: Integer] :String read GetTunerName;
    property TunerID  [idx: Integer] :String read GetTunerID;
    property Frequency  :Integer    read FFrequency;
    property BandWidth  :Integer    read FBandWidth;
  end;

implementation

// --------------------------------------------------------
//   vpeB֘A
// --------------------------------------------------------

function THDUS_Driver.GetTunerCount: Integer;
var
  cnt : Integer;
begin
  if Assigned(FTuner) then
  begin
    FTuner.get_TunerDevicesCount(cnt);
    Result := cnt;
  end
    else
  begin
    Result := -1;
  end;
end;

function THDUS_Driver.GetTunerName(idx: Integer): String;
var
  cnt : Integer;
  plg : IDCDVBTuningPluginDevice;
  prm : TDCDVBDeviceParams;
begin
  Result := '';
  if Assigned(FTuner) then
  begin
    FTuner.get_TunerDevicesCount(cnt);
    if (idx<cnt) then
    begin
      prm.Size := SizeOf(TDCDVBDeviceParams);
      FTuner.get_TunerDevice(idx,plg);
      plg.get_DeviceParams(prm);
      Result := prm.DeviceName;
    end;
  end;
end;

function THDUS_Driver.GetTunerID(idx: Integer): String;
var
  cnt : Integer;
  plg : IDCDVBTuningPluginDevice;
  prm : TDCDVBDeviceParams;
begin
  Result := '';
  if Assigned(FTuner) then
  begin
    FTuner.get_TunerDevicesCount(cnt);
    if (idx<cnt) then
    begin
      prm.Size := SizeOf(TDCDVBDeviceParams);
      FTuner.get_TunerDevice(idx,plg);
      plg.get_DeviceParams(prm);
      Result := prm.DeviceID;
    end;
  end;
end;

// --------------------------------------------------------
//   Constructor, Destructor
// --------------------------------------------------------

constructor THDUS_Driver.Create;
var
  GUID: TGUID;
begin
  FillChar(GUID,sizeof(GUID),0);
  FTuner     := TBDATuner.Create(GUID);
  FBDA       := nil;
  FLogMethod := nil;
  FTSMethod  := nil;
  FFrequency := 0;
  FBandWidth := 0;
  inherited;
end;

destructor THDUS_Driver.Destroy;
begin
  if Assigned(FBDA)   then FreeAndNil(FBDA);
  if Assigned(FTuner) then FreeAndNil(FTuner);
  inherited;
end;


// --------------------------------------------------------
//   Activate
// --------------------------------------------------------

function THDUS_Driver.Activate(ID:String): Boolean;
var
  i,p :Integer;
  cnt :Integer;
  plg :IDCDVBTuningPluginDevice;
  prm :TDCDVBDeviceParams;
begin
  Result := False;
  p := -1;
  if Assigned(FTuner) then
  begin
    FTuner.get_TunerDevicesCount(cnt);
    for i:=0 to cnt-1 do
    begin
      prm.Size := SizeOf(TDCDVBDeviceParams);
      FTuner.get_TunerDevice(i,plg);
      plg.get_DeviceParams(prm);
      if (prm.DeviceID = ID) then
      begin
        p := i;
        break;
      end;
    end;
    //
    if (p>=0) then
    begin
      // create BDA Device
      FTuner.get_TunerDevice(p,plg);
      plg.get_DeviceParams(prm);
      FBDA := TBDADevice.Create(prm);
      FBDA.put_ApplicationCallback(self);
      // Activate
      FBDA.put_ControlMessage(DCDVB_TUNING_PLUGIN_CTRL_ACTIVATE,0,0);
      Result := True;
    end;
  end;
end;

// --------------------------------------------------------
//   Deactivate
// --------------------------------------------------------

function THDUS_Driver.Deactivate: Boolean;
begin
  Result := False;
  if Assigned(FBDA) then
  begin
    // Deactivate
    FBDA.put_ControlMessage(DCDVB_TUNING_PLUGIN_CTRL_DEACTIVATE,0,0);
    Result := True;
  end;
end;

// --------------------------------------------------------
//   Tune
// --------------------------------------------------------

function THDUS_Driver.Tune(Freq:Integer; Band:Integer): Boolean;
var
  tune :TDCDVBTTuneRequest;
begin
  Result := False;
  if Assigned(FBDA) then
  begin
    // Tune
    tune.Size         := SizeOf(TDCDVBTTuneRequest);
    tune.DeviceType   := DCDVB_TUNING_PLUGIN_DEVICE_TYPE_DVBT;
    tune.FrequencyKHz := Freq;
    tune.BandwidthMHz := Band;
    FBDA.put_ControlMessage(DCDVB_TUNING_PLUGIN_CTRL_TUNEREQUEST,Integer(@tune),0);
    FFrequency        := Freq;
    FBandWidth        := Band;
    Result := True;
  end;
end;


// --------------------------------------------------------
//   Interface
// --------------------------------------------------------

function THDUS_Driver.ReceiveTSStream(ABuffer: PByte; ASize: Integer): Integer;
begin
  if Assigned(FTSMethod) then
  begin
    FTSMethod(ABuffer,ASize);
  end;
  Result := S_OK;
end;

function THDUS_Driver.LogEvent(AMethod: PChar; AText: PChar): Integer;
begin
  if Assigned(FLogMethod) then
  begin
    FLogMethod('['+String(AMethod)+'] '+String(AText));
  end;
  Result := S_OK;
end;


end.
